// ---------------------------------------------------------------------------

// STK500 programmer: UART module

// ---------------------------------------------------------------------------

#include "main.h"
#include "uart.h"

#define BAUD 115200
#define UBRRV (int)((F_CLK * 1E6)/(16.0 * BAUD) - 0.5)

// --------------------------- Init UART: ------------------------------------

void Uart_Init(void) 
{
  UBRRL = LO(UBRRV);
  UBRRH = HI(UBRRV);
  UCSRB = (1 << RXEN) | (1 << TXEN);   // enable tx/rx and no interrupt
  UCSRC = (1 << URSEL) | (3 << UCSZ0); // 8data, no parity, 1stop bit
}

// ------------------- Check for incoming byte: ------------------------------

bool Uart_Ready(void)
{
  return(UCSRA & (1 << RXC)); // check RX complete flag
}

// ------------------- Receive byte via UART: --------------------------------

char Uart_GetChar(void)  
{
  return(UDR);
}

// ---------------------- Send byte via UART: --------------------------------

void Uart_SendChar(char c) 
{
  while(!(UCSRA & (1 << UDRE))); // wait for empty transmit buffer
  UDR = c;
}

// -------------------- Send string via UART: --------------------------------

void Uart_SendStr(char *s) 
{
  while(*s)
  {
    Uart_SendChar(*s);
    s++;
  }
}

// -------------- Send string from flash via UART: ---------------------------

void Uart_SendStrFlash(char __flash *s)
{
  while(*s)
  {
    Uart_SendChar(*s);
    s++;
  }
}

// ---------------------- Send new line code: --------------------------------

void Uart_SendNewLine(void)
{
  Uart_SendChar('\r');
  Uart_SendChar('\n');
}

// ---------------------------------------------------------------------------
